import boto3
from datetime import datetime, timedelta, timezone
from pytargetingutilities.aws.s3.base_paginator import S3BasePaginator

class S3Iterator(S3BasePaginator):

    def __init__(self, pages, meta):
        self._ctx = 0
        super(S3Iterator,self).__init__(pages, meta)

    def __iter__(self):
        for page in self._pages:
            if 'Contents' in page:
                for item in page['Contents']:
                    self._keys.append(item)
            else:
                self._keys.append(page)
        return self

    def __next__(self):
        if self._ctx >= len(self._keys):
            raise StopIteration
        result = (
            boto3.resource('s3')
            .Object(self._meta['bucket'], self._keys[self._ctx]['Key'])
            .get()['Body']
            .read()
            .decode('utf-8')
            .strip()
        )
        self._ctx += 1
        return result

    def aggregate(self):
        ys = []
        for page in self:
            ys.append(page)
        return ys



import unittest

import pytest
from moto import mock_s3
from pytargetingutilities.aws.s3.helper import S3Helper


class TestS3Helper(unittest.TestCase):
    @mock_s3
    def test_get_latest_directory(self):
        pytest.create_bucket('test')
        mydates = [
            pytest.get_date(100),
            pytest.get_date(40),
            pytest.get_date(2),
            pytest.get_date(80),
        ]
        for idx, mydate in enumerate(mydates):
            pytest.add_dummy_data(
                'test', f'test/123456{idx}/{idx}.json', f'{idx}', mydate
            )
        latest_directory = S3Helper.get_latest_directory(
            'test', '', 'us-east-1'
        )
        self.assertEqual(latest_directory, 'test/1234562')
        for idx, mydate in enumerate(mydates):
            pytest.add_dummy_data(
                'test', f'newer/558{idx}/{idx}.json', f'{idx}', mydate
            )
        latest_directory = S3Helper.get_latest_directory(
            'test', 'test', 'us-east-1'
        )
        self.assertEqual(latest_directory, 'test/1234562')
        latest_directory = S3Helper.get_latest_directory(
            'test', '', 'us-east-1'
        )
        self.assertEqual(latest_directory, 'newer/5582')

    @mock_s3
    def test_object_newer_than_isnewer(self):
        pytest.create_bucket('test')
        pytest.add_dummy_data(
            'test', 'test/file/jay.son', "test", pytest.get_date(0)
        )
        older_date = pytest.get_date(10)
        self.assertTrue(S3Helper.object_newer_than(
            "test", "test/file/jay.son", older_date
        ))

    @mock_s3
    def test_object_newer_than_isolder(self):
        pytest.create_bucket('test')
        pytest.add_dummy_data(
            'test', 'test/file/jay.son', "test", pytest.get_date(10)
        )
        older_date = pytest.get_date(0)
        self.assertFalse(S3Helper.object_newer_than(
            "test", "test/file/jay.son", older_date
        ))

    @mock_s3
    def test_object_newer_than_sameage(self):
        pytest.create_bucket('test')
        pytest.add_dummy_data(
            'test', 'test/file/jay.son', "test", pytest.get_date(0)
        )
        older_date = pytest.get_date(0)
        self.assertFalse(S3Helper.object_newer_than(
            "test", "test/file/jay.son", older_date
        ))


if __name__ == '__main__':
    unittest.main()



import boto3
import logging


class EC2Helper:

    log = logging.getLogger()

    @staticmethod
    def start_instance(instance_config, instance_name):
        """Start instance by given config
        Args:
            instance_config (dict): ec2 config please see section run_instances
            in boto3 doc for more information's
            instance_name (str): name of instance

        Returns:
            [Instance]: instance object
        """
        instance = boto3.client('ec2').run_instances(
            **instance_config,
            TagSpecifications=[
                {
                    'ResourceType': 'instance',
                    'Tags': [
                        {
                            'Key': 'Name',
                            'Value': instance_name,
                        }
                    ],
                }
            ],
        )
        return instance

    @staticmethod
    def await_startup(instance_ids):
        """Wait until all instances of the list are in the status okstate

        Args:
            instance_ids (List[str]):: list of instance ids
        """
        waiter = boto3.client('ec2').get_waiter('instance_status_ok')
        waiter.wait(InstanceIds=instance_ids)

    @staticmethod
    def terminate_instances(instance_ids):
        """Terminate ec2 instances by list of instance ids

        Args:
            instance_ids (List[str]): list of instance ids
        """
        boto3.client('ec2').terminate_instances(InstanceIds=instance_ids)

    @staticmethod
    def create_image(instance_id, ami_name, ami_description):
        """Create image of running ec2 instance

        Args:
            instance_id (str): instance id
            ami_name (str): name of ami
            ami_description (str): description of ami

        Returns:
            Image: image object
        """
        return boto3.client('ec2').create_image(
            InstanceId=instance_id, Name=ami_name, Description=ami_description
        )

    @staticmethod
    def is_image_available(image_id):
        """Checks the image status and waits until the image status is no longer pending

        Args:
            name (str): name of ami image

        Returns:
            bool: true if available otherwise false
        """
        client = boto3.client('ec2')
        waiter = client.get_waiter('image_available')
        waiter.wait(Filters=[{'Name': 'image-id', 'Values': [image_id]}])
        image = client.describe_images(ImageIds=[image_id])
        if image['Images'][0]['State'] == 'available':
            EC2Helper.log.info(f'Image {image_id} has invalid state')
            return True
        EC2Helper.log.info(f'Image {image_id} has invalid state')
        return False

    @staticmethod
    def delete_ami_by_name(name):
        """Deletes ami image by name

        Args:
            name (str): name of ami image to delete

        Returns:
            bool: true if delete was successful otherwise false
        """
        client = boto3.client('ec2')
        images = client.describe_images(Owners=["self"])
        for image in images['Images']:
            if name == image['Name']:
                EC2Helper.delete_ami(image["ImageId"])
                return True
        EC2Helper.log.warning(
            f"""Couldn't find image with name {name}. The following images belongs to
             current user: {','.join(map(lambda x: x['Name'], images['Images']))}"""
        )
        return False

    @staticmethod
    def delete_ami(image_id):
        """Deletes ami image by image_id

        Args:
            image_id (str): id of ami image to delete

        Returns:
            bool: true if delete was successful otherwise false
        """
        client = boto3.client('ec2')
        EC2Helper.log.info(
            f'Deregister image {image_id}'
        )
        client.deregister_image(ImageId=image_id)
        return True

    @staticmethod
    def get_latest_image_id(starts_with=''):
        """Returns images id of newest ami
        Args:
            starts_with (str, optional): Filters image ids of account with startswith
            methode. Defaults to '' means filtering is disabled.

        Returns:
            str:  image id of latest ami
        """
        images = EC2Helper.get_image_ids(starts_with)
        if len(images) == 0:
            EC2Helper.log.warning("Didn't find any images in account")
            return None
        return images[-1]

    @staticmethod
    def get_image_ids(starts_with=''):
        """Returns images ids filtered with starts_with and sorted by date
        the last image id is the latest
        Args:
            starts_with (str, optional): Filters image ids of account with startswith
            methode. Defaults to '' means filtering is disabled.

        Returns:
            list[str]: list of image ids
        """
        client = boto3.client('ec2')
        images = client.describe_images(Owners=['self'])['Images']

        if starts_with != '':
            images = list(
                filter(lambda x: x['Name'].startswith(starts_with), images)
            )

        image_ids = list(
            map(
                lambda x: x['ImageId'],
                sorted(images, key=lambda img: img['CreationDate']),
            )
        )
        return image_ids



import unittest
from datetime import datetime, timedelta, timezone

import boto3
from moto import mock_s3
from pytargetingutilities.aws.s3.iterator import S3Iterator
import pytest


class TestS3Iterator(unittest.TestCase):
    @staticmethod
    def add_s3_data(item_length, bucket_name, region_name='us-east-1'):
        con = boto3.client('s3', region_name=region_name)
        con.create_bucket(Bucket=bucket_name)
        for idx in range(0, item_length):
            my_date = datetime.utcnow().replace(
                tzinfo=timezone.utc
            ) - timedelta(days=idx)
            pytest.add_dummy_data(
                bucket_name,
                f'crtt-profiling-{my_date.timestamp()}.json',
                f'testdata {idx}',
                my_date,
            )

    @mock_s3
    def test_aggregate_with_page_size_limit(self):
        item_length = 15
        bucket_name = 'test'
        self.add_s3_data(item_length, bucket_name)
        result = S3Iterator.paginator(bucket_name, max_items=1).aggregate()
        self.assertEqual(len(result), item_length)

    @mock_s3
    def test_aggregate_with_max_age(self):
        item_length = 15
        bucket_name = 'test'
        self.add_s3_data(item_length, bucket_name)
        my_iter = S3Iterator.date_paginator(bucket_name, 2).aggregate()
        result = [item for item in my_iter]
        self.assertTrue(len(result) >= 2)

    @mock_s3
    def test_paginator_with_file_ends_with(self):
        bucket_name = 'test'
        con = boto3.client('s3', region_name='us-east-1')
        con.create_bucket(Bucket=bucket_name)
        pytest.add_dummy_data(
            bucket_name,
            'file1.json',
            'test1',
            datetime.utcnow(),
        )
        pytest.add_dummy_data(
            bucket_name,
            'file2.json',
            'test2',
            datetime.utcnow(),
        )
        pytest.add_dummy_data(
            bucket_name,
            'file3.exe',
            'test3',
            datetime.utcnow(),
        )

        my_iter = S3Iterator.paginator(
            bucket_name, 2, ends_with='.json'
        ).aggregate()
        result = [item for item in my_iter]
        self.assertEqual(len(result), 2)

        my_iter = S3Iterator.paginator(
            bucket_name, 2, ends_with='.exe'
        ).aggregate()
        result = [item for item in my_iter]
        self.assertEqual(len(result), 1)

        my_iter = S3Iterator.date_paginator(
            bucket_name, 2, ends_with='.json'
        ).aggregate()
        result = [item for item in my_iter]
        self.assertEqual(len(result), 2)

        my_iter = S3Iterator.date_paginator(
            bucket_name, 2, ends_with='.exe'
        ).aggregate()
        result = [item for item in my_iter]
        self.assertEqual(len(result), 1)


if __name__ == '__main__':
    unittest.main()



from typing import List, Tuple, Union

import boto3

from pytargetingutilities.aws.s3.base_paginator import S3BasePaginator


class S3LineIterator(S3BasePaginator):
    """
    An iterator over all lines of all given s3 objects.
    Itarting over multiple objects is achieved by fixing the order of the objects and keeping track at which index of
    the iterator each object begins. Thus, if we take a slice out of our iterator, we know which lines to take of which
    objects.
    This is a 'nested' iterator, since we iterate over objects and inside the objects we iterate over lines. We identify
    the objects by its key. Thus, we call the start and stop of a slice of the iterator iter_start(stop) and the start
    and stop of a slice inside a object key_start(stop).
    """

    def __init__(self, pages, meta):
        super().__init__(pages, meta)
        self._key_len_map = None
        self.total_lines = None
        self._load_keys()
        self._set_key_len_map()

    def __len__(self) -> int:
        return self.total_lines

    def __getitem__(self, item: slice) -> List[str]:
        """ slice of lines from s3 objects; is only defined for slices without step [a:b:None], not subscript [a] """
        if not isinstance(item, slice):
            raise NotImplementedError('Only slicing is allowed, not subscripting; '
                                      'if you want to iterate over all lines, use [:]')
        if item.step is not None:
            raise NotImplementedError('Slicing with step is not implemented')
        if item.start is not None and item.start > self.__len__():
            raise IndexError('index is out of range')
        if item.start is not None and item.stop is not None and item.start >= item.stop:
            raise IndexError('start must be lesser than stop')
        key_slices = self._get_key_slices(item.start, item.stop)
        data = []
        for key_start_stop in key_slices:
            data.extend(self._object_get_lines(*key_start_stop))
        return data

    def _load_keys(self) -> None:
        """ load the keys of all objects in pages """
        for page in self._pages:
            if 'Contents' in page:
                for item in page['Contents']:
                    self._keys.append(item['Key'])
            else:
                self._keys.append(page['Key'])
        self._keys.sort()

    def _set_key_len_map(self) -> None:
        """ determine len of objects and the index of their first line in the context of the whole iterator """
        key_len_map = []  # list of tuples (key of object, number of lines in object, index of line 0 in iterator)
        iter_index = 0
        for key in self._keys:
            key_len = self._object_len(key)
            key_len_map.append((key, key_len, iter_index))
            iter_index += key_len
        self._key_len_map = key_len_map
        self.total_lines = iter_index

    def _object_len(self, key: str) -> int:
        i = -1  # in case there are no lines to iterate over
        for i, _ in enumerate(boto3.resource('s3')
                                   .Object(self._meta['bucket'], key)
                                   .get()['Body']
                                   .iter_lines()):
            pass
        return i + 1

    def _get_key_slices(self, iter_start: Union[int, None], iter_stop: Union[int, None]) -> List[Tuple[str, int, int]]:
        """
        for each key get the start and stop of the slice of the object that belong to the iterator slice defined by
        iter_start and iter_stop; if the key is not part of the slice, it is not in the returned list; key_start may
        be negative and key_stop may be greater than the object_len, both of which work as intended.
        Args:
            iter_start: start of iterator
            iter_stop: stop of iterator

        Returns:
            [(key, key_start, key_stop),]; only contains keys that are part of the slice
        """
        if iter_start is None:
            iter_start = 0
        if iter_stop is None:
            iter_stop = len(self)
        key_slices = []
        for key, object_len, line0_index in self._key_len_map:
            if iter_stop < line0_index or (iter_start >= (line0_index + object_len)):  # object not in iter slice
                continue
            key_start = iter_start - line0_index  # may be negative, which does not hurt
            key_stop = iter_stop - line0_index  # if iter_stop in later object, this is > object_len
            key_slices.append((key, key_start, key_stop))
        return key_slices

    def _object_get_lines(self, key: str, key_start: int, key_stop: int) -> List[str]:
        """ iterate over the object belonging to key and get the lines from start to stop """
        lines = []
        for index, line in enumerate(boto3.resource('s3')
                                          .Object(self._meta['bucket'], key)
                                          .get()['Body']
                                          .iter_lines()):
            if index < key_start:
                continue
            if index >= key_stop:
                break
            lines.append(line.decode('utf-8').strip())
        return lines

    def aggregate(self) -> List[str]:
        """ Return all lines of all objects """
        return [line for line in self[:]]



import unittest
from pytargetingutilities.tools.log import setup_logging, setup_logging_graylog
import logging


class TestLog(unittest.TestCase):

    def test_md5(self):
        logger = setup_logging("test", logging.INFO)
        self.assertIsNotNone(logger)
        try:
            logger.info('test')
        except:
            self.fail("logger.info raised Exception unexpectedly!")

        logger = setup_logging_graylog("test", logging.INFO, True, "", 0)
        self.assertIsNotNone(logger)
        try:
            logger.info('test')
        except:
            self.fail("logger.info raised Exception unexpectedly!")



import unittest
from pytargetingutilities.tools.hash import md5
import tempfile


class TestHash(unittest.TestCase):

    def test_md5(self):
        m_hash = md5('Pathtofilethatdoesnotexist.json')
        self.assertIsNone(m_hash)
        with tempfile.NamedTemporaryFile() as temp:
            temp.write(b'test data')
            temp.flush()
            my_hash = md5(temp.name)
            self.assertEqual(my_hash, 'eb733a00c0c9d336e65691a37ab54293')



import boto3
from datetime import datetime, timedelta, timezone


class S3BasePaginator(object):
    def __init__(self, pages, meta):
        self._pages = pages
        self._meta = meta
        self._keys = []

    @staticmethod
    def __paginator(bucket_name, directory, max_items, continuation_token):
        cfg = {
            'Bucket': bucket_name,
            'PaginationConfig': {'PageSize': max_items},
        }
        if directory:
            cfg['Prefix'] = directory

        if continuation_token:
            cfg['ContinuationToken'] = continuation_token

        return (
            boto3.client('s3').get_paginator('list_objects_v2').paginate(**cfg)
        )

    @staticmethod
    def __build_filter(filter_query):
        return "Contents[?" + " && ".join(filter_query) + "]"

    @classmethod
    def date_paginator(
            cls,
            bucket_name,
            max_age,
            max_items=1000,
            directory=None,
            ends_with=None,
            continuation_token=None,
    ):
        pages = cls.__paginator(
            bucket_name, directory, max_items, continuation_token
        )
        filter_date = (
            datetime.utcnow().replace(tzinfo=timezone.utc)
            - timedelta(days=max_age)
        ).strftime('%Y-%m-%d %H:%M:%S')
        query_filters = [
            f"to_string(LastModified)>='\"{filter_date}\"'"
        ]
        if ends_with:
            query_filters.append(f"ends_with(Key, `{ends_with}`)")
        querystr = cls.__build_filter(query_filters)

        return cls(
            pages.search(querystr),
            meta={
                'bucket': bucket_name,
                'max_items': max_items,
                'directory': directory,
            },
        )

    @classmethod
    def paginator(
            cls,
            bucket_name,
            max_items=1000,
            directory=None,
            ends_with=None,
            continuation_token=None,
    ):
        pages = cls.__paginator(
            bucket_name, directory, max_items, continuation_token
        )

        if ends_with:
            querystr = cls.__build_filter([f"ends_with(Key, `{ends_with}`)"])
            pages = pages.search(querystr)
        return cls(
            pages,
            meta={
                'bucket': bucket_name,
                'max_items': max_items,
                'directory': directory,
            },
        )


import unittest
from datetime import datetime, timedelta, timezone
from typing import List

from moto import mock_s3
import boto3
import pytest

from pytargetingutilities.aws.s3.line_iterator import S3LineIterator


class TestS3LineIterator(unittest.TestCase):
    @staticmethod
    def add_s3_data(item_lengths: List[int], bucket_name: str, region_name='us-east-1'):
        con = boto3.client('s3', region_name=region_name)
        con.create_bucket(Bucket=bucket_name)
        for idx, length in enumerate(item_lengths):
            my_date = datetime.utcnow().replace(
                tzinfo=timezone.utc
            ) - timedelta(days=len(item_lengths)-idx)
            data = '\n'.join([f'testdata {idx}, {l}' for l in range(length)])
            pytest.add_dummy_data(
                bucket_name,
                f'crtt-profiling-{my_date.timestamp()}.json',
                data,
                my_date,
            )

    @mock_s3
    def test_aggregate_one_file(self):
        item_length = [15]
        bucket_name = 'test'
        self.add_s3_data(item_length, bucket_name)
        result = S3LineIterator.paginator(bucket_name).aggregate()
        self.assertEqual(len(result), sum(item_length))

    @mock_s3
    def test_aggregate_two_files(self):
        item_length = [12, 3]
        bucket_name = 'test'
        self.add_s3_data(item_length, bucket_name)
        result = S3LineIterator.paginator(bucket_name).aggregate()
        self.assertEqual(len(result), sum(item_length))

    @mock_s3
    def test_len(self):
        item_length = [5, 3, 4, 8]
        bucket_name = 'test'
        self.add_s3_data(item_length, bucket_name)
        s3iter = S3LineIterator.paginator(bucket_name)
        self.assertEqual(len(s3iter), sum(item_length))

    @mock_s3
    def test_slicing_infile(self):
        item_length = [10]
        bucket_name = 'test'
        self.add_s3_data(item_length, bucket_name)
        lines = S3LineIterator.paginator(bucket_name)[2:5]
        is_ = [f'testdata {0}, {l}' for l in range(2, 5)]
        self.assertListEqual(lines, is_)

    @mock_s3
    def test_slicing_over_two_files(self):
        item_length = [10, 8]
        bucket_name = 'test'
        self.add_s3_data(item_length, bucket_name)
        lines = S3LineIterator.paginator(bucket_name)[5: 12]
        self.assertEqual(lines[0], 'testdata 0, 5')
        self.assertEqual(lines[-1], 'testdata 1, 1')

    @mock_s3
    def test_slicing_over_three_files(self):
        item_length = [10, 5, 5]
        bucket_name = 'test'
        self.add_s3_data(item_length, bucket_name)
        lines = S3LineIterator.paginator(bucket_name)[9:17]
        self.assertEqual('testdata 0, 9', lines[0])
        self.assertEqual('testdata 1, 0', lines[1])
        self.assertEqual('testdata 2, 1', lines[-1])

    @mock_s3
    def test_slicing_on_file_end(self):
        item_length = [2, 2]
        bucket_name = 'test'
        self.add_s3_data(item_length, bucket_name)
        lines = S3LineIterator.paginator(bucket_name)[1:2]
        self.assertListEqual(['testdata 0, 1'], lines)

    @mock_s3
    def test_slicing_on_file_start(self):
        item_length = [2, 2]
        bucket_name = 'test'
        self.add_s3_data(item_length, bucket_name)
        lines = S3LineIterator.paginator(bucket_name)[2:3]
        self.assertListEqual(['testdata 1, 0'], lines)

    @mock_s3
    def test_slicing_with_empty_file(self):
        item_length = [2, 0, 2]
        bucket_name = 'test'
        self.add_s3_data(item_length, bucket_name)
        lines = S3LineIterator.paginator(bucket_name)[1:3]
        self.assertListEqual(['testdata 0, 1', 'testdata 2, 0'], lines)

    @mock_s3
    def test_slicing_start_to_x(self):
        item_length = [4, 8]
        bucket_name = 'test'
        self.add_s3_data(item_length, bucket_name)
        lines = S3LineIterator.paginator(bucket_name)[:6]
        self.assertEqual(6, len(lines))

    @mock_s3
    def test_slicing_x_to_end(self):
        item_length = [4, 8]
        bucket_name = 'test'
        self.add_s3_data(item_length, bucket_name)
        lines = S3LineIterator.paginator(bucket_name)[6:]
        self.assertEqual(6, len(lines))

    @mock_s3
    def test_slicing_start_to_end(self):
        item_length = [4, 8]
        bucket_name = 'test'
        self.add_s3_data(item_length, bucket_name)
        lines = S3LineIterator.paginator(bucket_name)[:]
        self.assertEqual(sum(item_length), len(lines))

    @mock_s3
    def test_slicing_end_out_of_range(self):
        item_length = [4, 8]
        bucket_name = 'test'
        self.add_s3_data(item_length, bucket_name)
        lines = S3LineIterator.paginator(bucket_name)[10:15]  # that this works is consistent with list slicing
        self.assertEqual(2, len(lines))

    @mock_s3
    def test_slicing_indexing_error_start_out_of_range(self):
        item_length = [4, 8]
        bucket_name = 'test'
        self.add_s3_data(item_length, bucket_name)
        with self.assertRaises(IndexError):
            _ = S3LineIterator.paginator(bucket_name)[15:]

    @mock_s3
    def test_slicing_indexing_error_start_greater_stop(self):
        item_length = [4, 8]
        bucket_name = 'test'
        self.add_s3_data(item_length, bucket_name)
        with self.assertRaises(IndexError):
            _ = S3LineIterator.paginator(bucket_name)[5:2]

    @mock_s3
    def test_slicing_not_implemented_single_subscript(self):
        item_length = [4, 8]
        bucket_name = 'test'
        self.add_s3_data(item_length, bucket_name)
        with self.assertRaises(NotImplementedError):
            _ = S3LineIterator.paginator(bucket_name)[8]

    @mock_s3
    def test_slicing_not_implemented_iter_subscript(self):
        item_length = [4, 8]
        bucket_name = 'test'
        self.add_s3_data(item_length, bucket_name)
        with self.assertRaises(NotImplementedError):
            for _ in S3LineIterator.paginator(bucket_name):
                pass

    @mock_s3
    def test_slicing_not_implemented_ste(self):
        item_length = [4, 8]
        bucket_name = 'test'
        self.add_s3_data(item_length, bucket_name)
        with self.assertRaises(NotImplementedError):
            _ = S3LineIterator.paginator(bucket_name)[8:12:2]



import unittest

import pytest
from moto import mock_s3
from pytargetingutilities.aws.s3.helper import S3Helper
import json
from pytargetingutilities.tools.hash import md5_str


class TestS3Helper(unittest.TestCase):

    @mock_s3
    def test_write_with_hash(self):
        def write(output_path):
            with open(output_path, 'w') as f:
                json.dump({'TEST': 1}, f)
        pytest.create_bucket('test')
        S3Helper.write_with_hash('test', 'my_file.txt', write)
        my_file_content = S3Helper.read('test', 'my_file.txt')
        my_file_hash_content = S3Helper.read('test', 'my_file.txt.md5')
        self.assertEqual(my_file_content, b'{"TEST": 1}')
        self.assertEqual(my_file_hash_content.decode('utf-8'), md5_str('{"TEST": 1}'))

    @mock_s3
    def test_write_with_hash_with_hash_check(self):
        def write(output_path):
            with open(output_path, 'w') as f:
                json.dump({'TEST': 1}, f)
        pytest.create_bucket('test')
        S3Helper.write_with_hash('test', 'my_file.txt', write)
        my_file_content = S3Helper.read('test', 'my_file.txt')
        my_file_hash_content = S3Helper.read('test', 'my_file.txt.md5')
        self.assertEqual(my_file_content, b'{"TEST": 1}')
        self.assertEqual(my_file_hash_content.decode('utf-8'), md5_str('{"TEST": 1}'))
        hashed = S3Helper.write_with_hash('test', 'my_file.txt', write, True)
        self.assertTrue(hashed)

    @mock_s3
    def test_hash_check(self):
        def write(output_path):
            with open(output_path, 'w') as f:
                json.dump({'TEST': 1}, f)
        pytest.create_bucket('test')
        self.assertFalse(S3Helper.hash_check("test", "my_file.txt", write))


if __name__ == '__main__':
    unittest.main()



import time
from concurrent.futures.thread import ThreadPoolExecutor
from typing import List

import boto3
import botocore


class RunCommandEc2:

    max_pool_connections = 60

    @staticmethod
    def run_commands_and_wait(
        cmds: List[str],
        ec2_ids: List[str],
        delay,
        max_attempts,
        wait_for_invocation,
        timeout_seconds=60 * 60,
    ):
        response = RunCommandEc2.run_commands(ec2_ids, cmds, timeout_seconds)
        command_id = response['Command']['CommandId']
        waiter_args = [
            {
                'CommandId': command_id,
                'InstanceId': ec2_id,
                'PluginName': 'aws:RunShellScript',
                'WaiterConfig': {'Delay': delay, 'MaxAttempts': max_attempts},
            }
            for ec2_id in ec2_ids
        ]
        return RunCommandEc2._wait_for_result(
            command_id, ec2_ids, wait_for_invocation, waiter_args
        )

    @staticmethod
    def _wait_for_result(
        command_id, ec2_ids, wait_for_invocation, waiter_args
    ):
        waiter = boto3.client(
            'ssm',
            config=RunCommandEc2._create_config(
                RunCommandEc2.max_pool_connections
            ),
        ).get_waiter('command_executed')
        # we can only wait for one instance and to wait for all we need a pool
        with ThreadPoolExecutor(max_workers=len(waiter_args)) as executor:
            for arg in waiter_args:
                executor.submit(waiter.wait, **arg)
        outputs = RunCommandEc2._get_command_invocation(
            command_id, ec2_ids, wait_for_invocation
        )
        # wait until all tasks are completed
        if any(map(lambda x: x['Status'] == 'InProgress', outputs)):
            return RunCommandEc2._wait_for_result(
                command_id, ec2_ids, wait_for_invocation, waiter_args
            )
        return outputs

    @staticmethod
    def _get_command_invocation(command_id, ec2_ids, wait_for_invocation):
        ssm_client = boto3.client(
            'ssm',
            config=RunCommandEc2._create_config(
                RunCommandEc2.max_pool_connections
            ),
        )
        # sleep prevents for InvocationDoesNotExist error
        time.sleep(wait_for_invocation)
        outputs = [
            ssm_client.get_command_invocation(
                CommandId=command_id, InstanceId=ec2_id
            )
            for ec2_id in ec2_ids
        ]
        return outputs

    @staticmethod
    def run_commands(
        instance_ids: List[str], commands: List[str], timeout_seconds=60 * 60
    ):
        ssm_client = boto3.client(
            'ssm',
            config=RunCommandEc2._create_config(
                RunCommandEc2.max_pool_connections
            ),
        )
        response = ssm_client.send_command(
            InstanceIds=instance_ids,
            TimeoutSeconds=timeout_seconds,
            DocumentName='AWS-RunShellScript',
            Parameters={'commands': [' && '.join(commands)]},
        )
        return response

    @staticmethod
    def _create_config(max_pool_connections):
        return botocore.config.Config(
            max_pool_connections=max_pool_connections,
        )



import logging
import graypy
import sys


def setup_logging(
   name,
   loglevel
):
    """
    Configure a named logger
    Args:
        name (str): name of logger
        loglevel (int): log severity (DEBUG,INFO etc.)

    Returns:
    logger instance

    """
    return setup_logging_graylog(name, loglevel, False, "", 0)


def setup_logging_graylog(
   name,
   loglevel,
   use_graylog,
   graylog_host,
   graylog_port,
):
    """
    Configure a named logger
    Args:
        name (str): name of logger
        loglevel (int): log severity (DEBUG,INFO etc.)
        use_graylog (bool): enables graylog gelf logging
        graylog_host (str); IP/Host of graylog server
        graylog_port (int): Port of graylog server

    Returns:
    logger instance with graylog handler

    """
    logger = logging.getLogger(name)
    logformat = "[%(asctime)s] %(levelname)s:%(name)s:%(message)s"
    logging.basicConfig(
        level=loglevel,
        stream=sys.stdout,
        format=logformat,
        datefmt="%Y-%m-%d %H:%M:%S",
    )
    if use_graylog:
        handler = graypy.GELFUDPHandler(graylog_host, graylog_port)
        logger.addHandler(handler)
        logger.setLevel(loglevel)
    return logger



import datetime as dt
import json
import os
from tempfile import NamedTemporaryFile as tmp

import boto3
from botocore.exceptions import ClientError

from pytargetingutilities.aws.s3.iterator import S3Iterator
from pytargetingutilities.tools.hash import md5, md5_str


class S3Helper:
    @staticmethod
    def get_latest_directory(s3bucket, prefix='', region_name='eu-west-1'):
        if prefix is None:
            prefix = ''
        s3_client = boto3.client('s3', region_name=region_name)
        response = s3_client.list_objects_v2(Bucket=s3bucket, Prefix=prefix)
        if 'Contents' not in response:
            return None
        # Get the key of the object with the highest LastModified date
        latest_object = max(
            response['Contents'], key=lambda obj: obj['LastModified']
        )['Key']
        return os.path.dirname(latest_object)

    @staticmethod
    def merge_json(bucket, input_directory, output_directory, filename):
        s3_paginator = S3Iterator.paginator(
            bucket_name=bucket,
            directory=input_directory,
            ends_with='.json',
        )

        json_data = json.dumps(
            dict(
                (key, d[key])
                for d in [json.loads(content) for content in s3_paginator]
                for key in d
            )
        )

        hash_str = md5_str(json_data)

        boto3.resource('s3').Bucket('crtt-batch').objects.filter(
            Prefix=f'{output_directory}'
        ).delete()

        s3 = boto3.client('s3')
        s3.put_object(
            Bucket='crtt-batch',
            Key=f'{output_directory}{filename}',
            Body=json_data.encode(),
        )

        s3.put_object(
            Bucket='crtt-batch',
            Key=f'{output_directory}{filename}.md5',
            Body=hash_str.encode(),
        )

    @staticmethod
    def write_with_hash(
        bucket, file_path, writer_func, hash_check: bool = False
    ):
        with tmp() as map_file, tmp() as hash_file:
            writer_func(map_file.name)
            myhash = md5(map_file.name)
            if hash_check and S3Helper.__hash_check(bucket, file_path, myhash):
                return False
            hash_file.write(myhash.encode())
            hash_file.flush()
            S3Helper.__write(map_file.name, bucket, file_path)
            S3Helper.__write(hash_file.name, bucket, f'{file_path}.md5')
            return True

    @staticmethod
    def __hash_check(bucket, file_path, file_hash):
        """Returns true if hash has not changed"""
        # be agnostic whether file or its md5 file is in file_path
        file_path.rstrip(".md5")
        try:
            if S3Helper.read(bucket, f'{file_path}.md5') == file_hash:
                return True
            return False
        except ClientError as ex:
            if ex.response['Error']['Code'] == 'NoSuchKey':
                return False
            raise

    @staticmethod
    def hash_check(bucket, file_path, writer_func):
        """Returns true if hash of file on s3 has not changed"""
        with tmp() as map_file:
            writer_func(map_file.name)
            myhash = md5(map_file.name)
            return S3Helper.__hash_check(bucket, file_path, myhash)

    @staticmethod
    def write(bucket, file_path, writer_func):
        with tmp() as map_file:
            writer_func(map_file.name)
            S3Helper.__write(map_file.name, bucket, file_path)

    @staticmethod
    def __write(file_name, bucket, key):
        boto3.resource('s3').meta.client.upload_file(file_name, bucket, key)

    @staticmethod
    def read(bucket_name, key):
        s3 = boto3.client('s3')
        data = s3.get_object(Bucket=bucket_name, Key=key)
        return data['Body'].read()

    @staticmethod
    def download(bucket_name: str, key: str, destination: str):
        """
        Writes content of s3 object to file
        If the file already exists it is overwritten

        Args:
            bucket_name (str): name of the s3 bucket
            key (str): key of object in s3
            destination (str): path of file

        Returns:
            None - content of object is written to file
        """
        s3 = boto3.resource('s3')
        s3.Object(bucket_name, key).download_file(destination)

    @staticmethod
    def download_latest(
            bucket_name: str,
            key: str,
            destination: str,
            timestamp: dt.datetime
    ) -> bool:
        """
        Writes content of s3 object to file, if age of object is younger than
        timestamp

        Args:
            bucket_name (str): name of the s3 bucket
            key (str): key of object in s3
            destination (str): path of file
            timestamp (datetime): timestamp against which the age of the
                object is compared

        Returns:
            bool: whether file was written
        """
        if S3Helper.object_newer_than(bucket_name, key, timestamp):
            S3Helper.download(bucket_name, key, destination)
            return True
        return False

    @staticmethod
    def object_newer_than(
            bucket_name: str, key: str, timestamp: dt.datetime
    ) -> bool:
        """
        Returns true if object on s3 is younger than timestamp, else false

        Args:
            bucket_name (str): name of the s3 bucket
            key (str): key of object in s3
            timestamp (int): timestamp against which the age of the object is
                compared

        Returns:
            bool: True if object is younger, else false
        """
        s3 = boto3.resource('s3')
        last_modified = s3.Object(bucket_name, key).last_modified
        return last_modified > timestamp

    @staticmethod
    def upload_filtered_directory(
        bucket, prefix, local_directory, extension, delete_existing
    ):
        """Uploads local directory to s3 bucket.
        The local directory is filtered by given extension

        Args:
            bucket (str): s3 target bucket
            prefix (str): s3 upload location. Prefix must end with /
            local_directory (str): local directory
            extension (str): filter for local directory. If you want to upload only
            wheel packages set extension to .whl
            delete_existing (bool): deletes target directory if set to true
        """
        if not prefix.endswith('/'):
            raise AttributeError('Prefix must end with /')
        s3 = boto3.resource('s3')
        if delete_existing:
            s3.Bucket(bucket).objects.filter(Prefix=prefix).delete()
        list(
            map(
                lambda local_file: s3.meta.client.upload_file(
                    os.path.join(local_directory, local_file),
                    bucket,
                    f'{prefix}{local_file}',
                ),
                filter(
                    lambda x: x.endswith(extension),
                    os.listdir(local_directory),
                ),
            )
        )



"""
    Setup file for pytargetingutilities.
    Use setup.cfg to configure your project.

    This file was generated with PyScaffold 4.0.1.
    PyScaffold helps you to put up the scaffold of your new Python project.
    Learn more under: https://pyscaffold.org/
"""
import os
import re
from setuptools import setup

VERSION = '0.0.0'

with open('src/pytargetingutilities/__init__.py', 'r') as fd:
    VERSION = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', fd.read(), re.MULTILINE
    ).group(1)

requirements_file = os.path.join(os.path.dirname(__file__), 'requirements.txt')
with open(requirements_file) as f:
    requirements = f.read().splitlines()

if __name__ == "__main__":
    try:
        setup(version=VERSION, install_requires=requirements)
    except:  # noqa
        print(
            "\n\nAn error occurred while building the project, "
            "please ensure you have the most updated version of setuptools, "
            "setuptools_scm and wheel with:\n"
            "   pip install -U setuptools setuptools_scm wheel\n\n"
        )
        raise



import hashlib


def md5(file_path):
    hash_md5 = hashlib.md5()
    try:
        with open(file_path, 'rb') as f:
            for chunk in iter(lambda: f.read(4096), b''):
                hash_md5.update(chunk)
        return hash_md5.hexdigest()
    except FileNotFoundError:
        return None

def md5_str(text):
    hash_object = hashlib.md5(text.encode())
    return hash_object.hexdigest()
